require 'rails'

module ProductManagement
  class Engine < Rails::Engine
    isolate_namespace ProductManagement

    config.to_prepare do
      # Load concerns
      Dir.glob(ProductManagement::Engine.root + "app/model/concerns/*.rb").each do |concern|
        require_dependency(concern)
      end
    end

    config.generators do |g|
      g.test_framework :rspec, fixture_replacement: :factory_bot, views: false, helper: true
      g.view_specs false
      g.helper_specs true
      g.fixture_replacement :factory_bot, dir: 'spec/factories'
    end

    # config.autoload_paths << File.expand_path("../validators", __FILE__)

    # Load translations from config/locales/*.rb,yml are auto loaded.
    config.i18n.load_path += Dir[File.join(File.dirname(__FILE__), '..', '..', 'config', 'locales', '**', '*.{rb,yml}').to_s]

    # monkey patch via: http://pivotallabs.com/leave-your-migrations-in-your-rails-engines/
    #                   https://content.pivotal.io/blog/leave-your-migrations-in-your-rails-engines
    #                   https://blogs.vmware.com/tanzu/leave-your-migrations-in-your-rails-engines
    initializer "product_management.migrations" do |app|
      unless app.root.to_s.match root.to_s
        # puts "Loading migrations from #{self.class.name} engine"
        config.paths['db/migrate'].expanded.each { |expanded_path| app.config.paths['db/migrate'] << expanded_path }
      end
      # puts "#{self.class.name} engine migrations loaded"
      # puts app.config.paths['db/migrate']
    end

    initializer "product_management.factories", after: "factory_bot.set_factory_paths" do
      if Rails.env.test?
        puts "Loading factories from #{self.class.name} engine"
        FactoryBot.definition_file_paths << File.expand_path('../../spec/factories', __dir__) if defined?(FactoryBot)
      end
    end

    # Execute local initializers for any in-house gems
    initializer "product_management.load_config_initializers", after: :load_config_initializers do |_app|
      Rails.logger.info "ProductManagement::Engine : Loading initializers"
      require File.join(File.dirname(__FILE__), '..', '..', 'config', 'in_house_gems')
      in_house_engines = PRODUCT_MANAGEMENT_IN_HOUSE_ENGINES
      in_house_engines.each do |in_house_engine|
        initializer = File.expand_path(File.join(File.dirname(__FILE__), '..', '..', 'config', 'initializers',
                                                 "#{in_house_engine[:name]}.rb"))
        Rails.logger.info "ProductManagement::Engine : Loading '#{initializer}'"
        require initializer if File.exist? initializer
      end
    end

    # Add the engine's asset paths to the application's load path
    initializer "product_management.assets" do |app|
      app.config.assets.paths << root.join("app/assets/stylesheets")
      app.config.assets.paths << root.join("app/javascript")
      app.config.assets.precompile += %w[ product_management_manifest ]
    end
  end
end
